﻿Imports System.IO
Imports System.Net
Imports System.Net.Mail
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Security.Permissions

Public Class SendFileForm


#Region "アプリケーションの定数"

    Private Const FILE_FORMAT = "yyyyMMddHHmmssfff"
    Private Const INI_NAME = "SENDFILE.INI"

#End Region

#Region "APIの定数"

    Public Const WM_KEYDOWN As Integer = &H100
    Public Const WM_KEYUP As Integer = &H101

    Public Const WH_KEYBOARD_LL As Integer = 13

    Public Const WS_EX_TOOLWINDOW As Int32 = &H80
    Public Const WS_POPUP As Int32 = &H80000000
    Public Const WS_VISIBLE As Int32 = &H10000000
    Public Const WS_SYSMENU As Int32 = &H80000
    Public Const WS_MAXIMIZEBOX As Int32 = &H10000

#End Region


#Region "カスタムイベント"
    Public Event GKeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
    Public Event GKeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
#End Region


#Region "ユーザ定義の構造体"

    ''' <summary>
    ''' INIファイルの内容
    ''' </summary>
    Public Structure AppIniStruct

        Public computername As String
        Public filepath As String

        Public host As String
        Public port As Integer
        Public user As String
        Public password As String

        Public from As String
        Public [to] As String
        Public subject As String
        Public body As String

    End Structure

#End Region

    ''' <summary>
    ''' アプリケーションの動作設定
    ''' </summary>
    Private INI As AppIniStruct


#Region "API"

#Region "INI"

    <DllImport("KERNEL32.DLL")>
    Public Shared Function WritePrivateProfileString(
                                                    ByVal lpAppName As String,
                                                    ByVal lpKeyName As String,
                                                    ByVal lpString As String,
                                                    ByVal lpFileName As String) As Integer
    End Function

    <DllImport("KERNEL32.DLL", CharSet:=CharSet.Auto)>
    Public Shared Function GetPrivateProfileString(
                                                  ByVal lpAppName As String,
                                                  ByVal lpKeyName As String,
                                                  ByVal lpDefault As String,
                                                  ByVal lpReturnedString As String,
                                                  ByVal nSize As Integer,
                                                  ByVal lpFileName As String) As Integer
    End Function

    <DllImport("KERNEL32.DLL", CharSet:=CharSet.Auto)>
    Public Shared Function GetPrivateProfileInt(
                                               ByVal lpAppName As String,
                                               ByVal lpKeyName As String,
                                               ByVal nDefault As Integer,
                                               ByVal lpFileName As String) As Integer
    End Function

#End Region


#End Region

    ''' <summary>
    ''' 最終チェック
    ''' </summary>
    Private lastCheck = DateTime.Now
    ''' <summary>
    ''' チェック中フラグ
    ''' </summary>
    Private checking = False

#Region "インスタンス変数・共有変数"


#End Region

    ''' <summary>
    ''' Loadイベントで初期化
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub SendFileForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Setup()
        TmrFileCheck.Start()
    End Sub

    Private Sub 新しいファイルのチェックToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles 新しいファイルのチェックToolStripMenuItem.Click
        CheckFile()
    End Sub

    Private Sub 終了ToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles 終了ToolStripMenuItem.Click
        Me.Close()
    End Sub


    ''' <summary>
    ''' CreateParamsをオーバーライドしウィンドウを非表示状態にする
    ''' </summary>
    ''' <returns></returns>
    Protected Overrides ReadOnly Property CreateParams() As CreateParams
        <SecurityPermission(SecurityAction.Demand,
        Flags:=SecurityPermissionFlag.UnmanagedCode)>
        Get

            Dim cp As System.Windows.Forms.CreateParams
            cp = MyBase.CreateParams
            cp.ExStyle = WS_EX_TOOLWINDOW
            cp.Style = WS_POPUP Or WS_VISIBLE Or
            WS_SYSMENU Or WS_MAXIMIZEBOX
            cp.Height = 0
            cp.Width = 0
            Return cp
        End Get
    End Property

    ''' <summary>
    ''' INIファイルから動作設定を読み込む
    ''' </summary>
    Private Sub Setup()

        Dim inipath = System.IO.Path.Combine(Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath), INI_NAME)

        INI.computerName = GetIni("DEFAULT", "COMPUTER_NAME", "", inipath)
        INI.filepath = GetIni("DEFAULT", "FILE_PATH", "", inipath)

        INI.host = GetIni("MAIL", "SMTP_HOST", "", inipath)
        INI.port = GetPrivateProfileInt("MAIL", "SMTP_PORT", 25, inipath)
        INI.user = GetIni("MAIL", "SMTP_USER", "", inipath)
        INI.password = GetIni("MAIL", "SMTP_PASSWORD", "", inipath)
        INI.from = GetIni("MAIL", "FROM_ADDRESS", "", inipath)
        INI.[to] = GetIni("MAIL", "TO_ADDRESS", "", inipath)
        INI.subject = GetIni("MAIL", "SUBJECT", "", inipath)
        INI.body = GetIni("MAIL", "BODY", "", inipath)

    End Sub

    ''' <summary>
    ''' タイマーのイベントハンドラ
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Timer1_Tick(sender As Object, e As EventArgs) Handles TmrFileCheck.Tick
        CheckFile()
    End Sub

    ''' <summary>
    ''' 新しいファイルのチェックと送信
    ''' </summary>
    Private Sub CheckFile()

        ' チェック中の場合は処理を飛ばす
        If checking Then
            Exit Sub
        End If

        checking = True

        Try

            Dim now = DateTime.Now

            ' 新しいファイルのリストを取得し、1件以上取得できた場合は送信確認ダイアログを表示する
            Dim fileList = getNewFiles(lastCheck)
            If fileList.Count >= 1 Then

                ' 送信前の確認
                Dim msg = "送信フォルダに" & CStr(fileList.Count) & "個のファイルを見つけました。" & vbNewLine &
                    "送信しますか?" & vbNewLine &
                    "(送信先:" & INI.to & ")"
                If MessageBox.Show(msg, "ファイル送信の確認", MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) <> DialogResult.OK Then
                    Exit Sub
                End If

                ' メールで送信
                SendMail(fileList)
            End If

        Finally
            lastCheck = Now
            checking = False
        End Try

    End Sub

    ''' <summary>
    ''' 基準日時よりも新しいファイルのリストを取得する
    ''' (更新日時ベースでチェック)
    ''' </summary>
    ''' <param name="base"></param>
    ''' <returns></returns>
    Private Function getNewFiles(base As DateTime) As List(Of String)

        Dim list As New List(Of String)

        For Each file1 In Directory.EnumerateFiles(INI.filepath)
            If File.GetLastAccessTime(file1) > base Then
                list.Add(file1)
            End If
        Next

        Return list

    End Function


    ''' <summary>
    ''' メール送信
    ''' </summary>
    ''' <param name="fileList"></param>
    Private Sub SendMail(fileList As List(Of String))

        ' 本文を構築
        Dim body As New System.Text.StringBuilder
        body.Append(Microsoft.VisualBasic.Replace(INI.body, "\n", Microsoft.VisualBasic.vbNewLine)).Append(vbNewLine)
        body.Append(INI.computername).Append(vbNewLine)

        Using msg = New System.Net.Mail.MailMessage(INI.from, INI.[to], INI.subject, body.ToString)

            ' 対象ファイルの存在確認をしながら、存在するファイルを添付
            For Each path In fileList
                If Not File.Exists(path) Then
                    Continue For
                End If
                msg.Attachments.Add(New Attachment(path))
            Next

            ' 送信ファイルがない場合は処理終了
            If msg.Attachments.Count = 0 Then
                Exit Sub
            End If

            ' SMTPクライアントを構築しメール送信
            Using client = New SmtpClient(INI.host, INI.port)
                client.DeliveryMethod = SmtpDeliveryMethod.Network
                client.Credentials = New NetworkCredential(INI.user, INI.password)
                client.Send(msg)
            End Using
        End Using

    End Sub

#Region "ユーティリティ機能"

    ''' <summary>
    ''' INIファイルから文字列の取得
    ''' </summary>
    ''' <param name="appname"></param>
    ''' <param name="keyname"></param>
    ''' <param name="defaultvalue"></param>
    ''' <param name="filename"></param>
    ''' <returns></returns>
    Private Shared Function GetIni(ByVal appname As String, ByVal keyname As String, ByVal defaultvalue As String, ByVal filename As String) As String
        Dim sb As String = Space(255)
        GetPrivateProfileString(appname, keyname, defaultvalue, sb, Len(sb), filename)
        Return Microsoft.VisualBasic.Left(sb, InStr(sb, Chr(0)) - 1).ToString
    End Function

#End Region

End Class
